/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.NewRouting
 *
 * de.bsvrz.kex.tls.osi7.NewRouting is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.NewRouting is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.NewRouting.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.NewRouting.
 *
 *  de.bsvrz.kex.tls.osi7.NewRouting ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.NewRouting wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7;

import de.bsvrz.sys.funclib.debug.Debug;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * Klasse,ber die das genderte Routing initialisiert wird.
 *
 * @author inovat
 * @author Dipl.-Ing. Hans Chrstian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */

public class NewRouting {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/**
	 * DebugLogger fr Debug-Ausgaben
	 */
	private static final Debug debug = Debug.getLogger();

	private static NewRouting newRouting;
// Private ----------------------------------------------------------------
		/**
	 * Map, in der Zieladressen umgeroutet werden knnen, Schlssel ist ursprungliche Knotennummer, Objekt ist neue
	 * Zielknotennummer. Wird u. a. fr Schaltvorgaben von neuer an alte VRZ verwendet, um Schaltvorgaben nicht direkt an
	 * die DE sondern an die UZ zu schicken.
	 */
	private Map<Integer, Integer> newRoutingMap = new HashMap<Integer, Integer>();
	private int[][] _newRouting;

// ------------------------------------------------------------------------
// STATISCHE METHODEN
// ------------------------------------------------------------------------

	public synchronized static NewRouting getInstance() {
		if (newRouting == null) {
			newRouting = new NewRouting();
		}
		return newRouting;
	}

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Construktor, tut nichts. Singelton muss ber Aufruf von {@link #getInstance().#init(String)} initialisiert werden.
	 */
	private NewRouting() {
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Liefert die Zieladresse (OSI7) unter Bercksichtung eventuell versorgter neuer Zieladressen zurck.
	 *
	 * @param destination Ursprngliche Zieladresse.
	 *
	 * @return Neue Zieladresse.
	 */
	public int getDestination(int destination) {
		// Prfen, ob Zieladresse umgeroutet werden muss.
		if(newRoutingMap.containsKey(destination)){
			destination = newRoutingMap.get(destination);
		}
	 return destination;
	}

	/**
	 * Initialisiert die Klasse, indem die bergebene Klasse dynamische geladen wird.
	 *
	 * @param newRoutingClassName Klassenname, der die Liste mit den umzusetzenden OSI7-Routing Informationen enthlt.
	 */
	public void init(String newRoutingClassName) {
		Class classNewRouting;
		try {
			classNewRouting = Class.forName(newRoutingClassName);
			/**
			 * Methode im UmsetzungsModul, welche dynamisch zur Laufzeit ermittelt wird.
			 */
			Method method = classNewRouting.getMethod("getNewRouting", new Class[]{});
			_newRouting = (int[][]) method.invoke(null, new Object[]{});
		}
		catch (ClassNotFoundException e) {
			debug.error("Klasse zur Auflsung externer Objektreferenzen konnte nicht geladen werden: " + newRoutingClassName, e);
		}
		catch (NoSuchMethodException e) {
			debug.error("Methode 'getNewRouting()' in der Klasse '" + newRoutingClassName + "' existiert nicht.", e);
		}
		catch (IllegalAccessException e) {
			debug.error("Fehler beim Zugriff auf die Methode 'getSourceDataReferences()' in der Klasse '" + newRoutingClassName + "'.", e);
		}
		catch (InvocationTargetException e) {
			debug.error("InvocationTargetException:", e);
		}

			/** Initialisiert die Tabelle fr das Umrouten von Zielen. */
	for(int[] osi7Adressen: _newRouting){
		newRoutingMap.put(osi7Adressen[0], osi7Adressen[1]);
	}
	}
}