/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.Fg003Typ077
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg003Typ077 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Typ077 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.Fg003Typ077.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.Fg003Typ077.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Typ077 ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg003Typ077 wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */



package de.bsvrz.kex.tls.osi7.conversion;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.kex.tls.osi7.*;
import de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung;
import de.bsvrz.sys.funclib.debug.Debug;

//~ KLASSEN ===================================================================

/**
 * Umsetzungsklasse fr FG 3, Typ 77 Erweiterung: untersttzt auch nachgelieferte Telegramme (Standard ID+32)
 *
 * Diese Klasse basiert auf einer Klasse der Firma inovat um das Schema zu erhalten.
 *
 * @version $Revision$ / $Date$ / ($Author$)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Thomas Drapp
 */
public class Fg003Typ077 extends Fg003Default {

    // ------------------------------------------------------------------------
    // VARIABLENDEKLARATIONEN
    // ------------------------------------------------------------------------
    // Sonstige ---------------------------------------------------------------

    /** DebugLogger fr Debug-Ausgaben */
    private static final Debug debug = Debug.getLogger();

    // Private ----------------------------------------------------------------

    //~ FELDER ================================================================

    /**
     * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "1", "29", "2", "typ.deLve"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Fr jede Attributguppen/Aspekt Kombination erfolgt eine
     * Empfangsanmeldung als Empfnger.</li> <li> In der Methode {@link #update} wird fr jede zur
     * Attributgruppen/Aspket-Kombination passende Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag
     * wie folgt gebildet wird (hier fr den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br>
     * Dabei enthlt <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li> <li> Fr jede
     * Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfnger. Sind mehrere passende Eintrage
     * vorhanden, werden entsprechend mehrere Methodenaufrufe durchgefhrt. Dadurch knnen mehrere ATG/ASP-Kombinationen in
     * einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eines Fachmodells):</li>
     * <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] davToTls = new Object[][] {

        // Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
    };

    /**
     * Tabelle, in der das Verhalten beim Empfang von TLS De-Blcken eingetragen wird. Jede Tabellenzeile hat folgenden
     * Aufbau:
     * <pre>
     * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
     * z.B.
     * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "1", "29", "2", "typ.deLve"
     * </pre>
     * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Fr jede Attributguppen/Aspekt Kombination erfolgt eine
     * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird fr jede zum De-Block passende
     * Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt gebildet wird (hier fr
     * den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock, Data,
     * DataDescription)</code><br> Dabei enthlt der <code>deBlock</code> die Daten des zu verarbeitenden De-Blocks und
     * <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem direkt die Inhalte des
     * De-Blocks zugewiesen werden knnen.<code>DataDescription</code> enthlt die passende {@link
     * de.bsvrz.dav.daf.main.DataDescription} fr die zu sendenden Daten.</li> <li> Fr jede Attributguppen/Aspekt
     * Kombination erfolgt eine Sendeanmeldung als Quelle. Sind mehrere passende Eintrage vorhanden, werden entsprechend
     * mehrere Methodenaufrufe durchgefhrt. Dadurch kann ein De-Block mehrfach ausgewert und an verschiedene
     * Attributgruppen weitergerleitet werden (z.B. protokollnahe TLS-Attributgruppe und Attributgruppe eine
     * Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt immer). </ul>
     */
    private final Object[][] tlsToDav = new Object[][] {
        { "atg.tlsUfdErgebnisMeldungGriffigkeitGR", "asp.tlsAntwort", "3", "77", "4", "typ.deUfd" }, { "atg.tlsUfdErgebnisMeldungGriffigkeitGR", "asp.tlsAntwort", "3", "77", "36", "typ.deUfd" }, { "atg.ufdsGriffigkeit", "asp.externeErfassung", "3", "77", "4", "typ.ufdsGriffigkeit" }
    };

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    // ------------------------------------------------------------------------
    // KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
    // ------------------------------------------------------------------------

    /** Objekterzeugung mit Anmeldung der globalen und lokalen Umsetzungseintrge */
    public Fg003Typ077(ClientDavInterface daf, Eak eak, De de) {
        super(daf, eak, de);
        subscribeAllData(tlsToDav, davToTls);
    }

    //~ METHODEN ==============================================================

    /**
     * FG 3 : Typ: 77 Id: 4 Name: UFD-Griffigkeit GR (Antwort) TLS-Datenbeschreibung:
     * <pre>
     * Format                 Bereich                   Auflsung                     Codierung
     * 8 bit                  0,01...1,00               0,01                          0,00 = 0d = 00h
     *                                                                                1,00 = 100d = 64h
     *                                                                                nicht ermittelbar = 255 = FFh
     *
     * Es ist zu beachten, dass in Relation zur Strecke die Messung immer nur punktfrmig sein kann und daher die
     * Ergebnisse entsprechend interpretiert werden mssen.<br>
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ077Id004AtgTlsUfdErgebnisMeldungGriffigkeitGRAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = null;

        // _Messwert_ aus Telegramm ------------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        data.getUnscaledValue("Griffigkeit").set(_Messwert_);

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.deUfd", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // Ergebnisdatensatz erzeugen...
        resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);

        return resultData;
    }

    /**
     * Umsetzung von TLS-DE-Block in logischen Umfelddatensensor.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ077Id004AtgUfdsGriffigkeitAspExterneErfassung(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // DE-Daten-Ausfallberwachung aufrufen und nchsten Zeitpunkt eintragen, an dem sptesten KEINE DATEN gesendet wird.
        DeAusfallUeberwachung.addAuftrag(_config, _de, dataDescription, "typ.ufdsGriffigkeit", _eakConversionModul.getTimeStamp(), _intervallDauerUFD);

        // _Messwert_ aus Telegramm -------------------------------------------------------------------------------
        int _Messwert_ = deBlock.getDeData().readUnsignedByteAsInt();

        return tlsDatenNachResultUfdLogisch(_Messwert_, data, dataDescription, "Griffigkeit", 255, _intervallDauerUFD);
    }

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /**
     * Entspricht {@link de.bsvrz.kex.tls.osi7.conversion.Fg003Typ077#Fg003Typ077Id004AtgTlsUfdErgebnisMeldungGriffigkeitGRAspTlsAntwort(de.bsvrz.kex.tls.osi7.DeBlock,
     * de.bsvrz.dav.daf.main.Data, de.bsvrz.dav.daf.main.DataDescription)}, jedoch werden die Daten als nachgeliefert gekennzeichnet.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz fr die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung fr die Attributgruppe/Aspekt-Kombination.
     *
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg003Typ077Id036AtgTlsUfdErgebnisMeldungGriffigkeitGRAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {

        // Generierte Methode: Standardverarbeitung wie bei Id 4, aber als "nachgelieferte" Daten an den DaV bergeben.
        ResultData resultData = Fg003Typ077Id004AtgTlsUfdErgebnisMeldungGriffigkeitGRAspTlsAntwort(deBlock, data, dataDescription);

        return new ResultData(_de.getDe(), dataDescription, resultData.getDataTime(), resultData.getData(), true);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 01.03.13
