/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung
 *
 * de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung ist Freie Software: Sie knnen es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder spteren
 *  verffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.deausfallueberwachung.DeAusfallUeberwachung wird in der Hoffnung, dass es ntzlich sein wird, aber
 *  OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License fr weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */


package de.bsvrz.kex.tls.osi7.deausfallueberwachung;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.kex.tls.TlsApplication;
import de.bsvrz.kex.tls.osi7.De;
import de.bsvrz.sys.funclib.debug.Debug;

//~ KLASSEN ===================================================================

/**
 * Implementiert eine Ausfallberwachung fr nicht zyklisch erhaltene Telegramme.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class DeAusfallUeberwachung extends Thread {

    /**
     * DebugLogger fr Debug-Ausgaben.
     */
    private static final Debug debug = Debug.getLogger();

    /**
     * Fuegt einen neuen Ueberwachungsauftrag fuer eine spezifische OT/ATG/ASP, ausgeloest auf Basis von empfangenen
     * TLS-Daten an einer spezifischen DE, ein. Aeltere Auftrage fuer eine solche Datenidentifikation werden
     * automatisch ersetzt, so dass je Datenidentifikation maximal ein Ueberwachungsauftrag anliegt.
     *
     * @param konfig                   Das Objekt zum Zugriff auf die Konfiguration
     * @param de                       Das aktuelle DE, welches ueberwacht wird
     * @param datenBeschreibung        Die Datenbeschreibung (ATG/ASP), die ueberwacht wird
     * @param pidObjektTyp             Die PID des Objekttyps (passend zur datenBeschreibung), die ueberwacht wird. Dies kann
     *                                 z. B. der Fahrstreifen sein, der auf die ueberwachte DE referenziert.
     * @param intervallBeginn          Der Intervallbeginn der aktuellen gueltigen Daten in ms seit dem 01.01.1970
     * @param intervallDauerInSekunden Die aktuell gueltige Intervalldauer fuer diese Daten. Aus Intervallbeginn,
     *                                 Intervalldauer und der AnzahlIntervalleFuerDatenAusfallErkennung wird der
     *                                 Zeitpunkt ermittelt, an dem spaetestens der naechste gueltige Datensatz
     *                                 empfangen werden muss. Sonst wird ein leerer Datensatz verschickt.
     */
    public static void addAuftrag(DataModel konfig, De de, DataDescription datenBeschreibung, String pidObjektTyp, long intervallBeginn, long intervallDauerInSekunden) {

        // Auftrag nur erzeugen, wenn Ueberwachungsintervallanzahl > 0 ist
        if (TlsApplication.getAnzahlIntervalleFuerDatenAusfallErkennung() > 0) {
            Auftrag auftrag = new Auftrag(konfig, de, datenBeschreibung, pidObjektTyp, intervallBeginn, intervallDauerInSekunden);
            AuftragsListe auftragsListe = AuftragsListe.getInstanz();

            auftragsListe.addAuftrag(auftrag);
        }
    }


    public void run() {
        final long WARTEZEIT_AUF_NAECHSTEN_AUFTRAG = 5*1000;
        debug.info(String.format("%n=================================================================%n" + "DeAusfallUeberwachung gestartet...%n" + "=================================================================%n"));

        // Auftragsliste holen
        AuftragsListe auftragsListe = AuftragsListe.getInstanz();

        // Endlosschleife mit Prfung der Auftragsliste.
        while (true) {
            Auftrag auftrag;

            if ((auftrag = auftragsListe.holeNaechstenAuftrag()) != null) {
                auftrag.ausfuehren();
                debug.info(String.format("Ausgefhrter Auftrag: %s", auftrag));
            } else {

                // Zurzeit ist nichts zu tun, also warten und es dann noch mal versuchen...
                debug.fine(String.format("Zur Zeit keine sofort auszufhrenden Auftrge in der Auftragsliste. Warte %d Sekunden.", WARTEZEIT_AUF_NAECHSTEN_AUFTRAG / 1000));
                try {
                    Thread.sleep(WARTEZEIT_AUF_NAECHSTEN_AUFTRAG);
                }
                catch (InterruptedException e) {
                    debug.info("Sleep der Auftragsbearbeitung der DeAusfallUeberwachung wurde vorzeitig unterbrochen: ", e);
                }
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 07.03.12
